@@ -83,7 +83,7 @@ GLuint compile_shader(const string &shader_src, GLenum type)
 		}
 
 		fprintf(stderr, "Failed to compile shader:\n%s\n", src_with_lines.c_str());
-		exit(1);
+		abort();
 	}
 
 	return obj;
@@ -101,7 +101,7 @@ GLuint link_program(GLuint vs_obj, GLuint fs_obj)
 		GLchar error_log[1024] = { 0 };
 		glGetProgramInfoLog(program, 1024, nullptr, error_log);
 		fprintf(stderr, "Error linking program: %s\n", error_log);
-		exit(1);
+		abort();
 	}
 	return program;
 }
