@@ -7,6 +7,7 @@
 #include "player.h"
 #include "futatabi_midi_mapping.pb.h"
 #include "midi_mapping_dialog.h"
+#include "pbo_pool.h"
 #include "shared/aboutdialog.h"
 #include "shared/disk_space_estimator.h"
 #include "shared/post_to_main_thread.h"
@@ -324,7 +325,11 @@ void MainWindow::change_num_cameras()
 		display->setAutoFillBackground(true);
 		layout->addWidget(display);
 
-		display->set_overlay(to_string(i + 1));
+		if (global_flags.source_labels.count(i + 1)) {
+			display->set_overlay(global_flags.source_labels[i + 1]);
+		} else {
+			display->set_overlay(to_string(i + 1));
+		}
 
 		QPushButton *preview_btn = new QPushButton(this);
 		preview_btn->setMaximumSize(20, 17);
