@@ -271,7 +271,7 @@ ALSAPool::ProbeResult ALSAPool::probe_device_once(unsigned card_index, unsigned
 	reset_device(internal_dev_index);  // Restarts it if it is held (ie., we just replaced a dead card).
 
 	DeviceSpec spec{InputSourceType::ALSA_INPUT, internal_dev_index};
-	global_audio_mixer->set_display_name(spec, display_name);
+	global_audio_mixer->set_device_parameters(spec, display_name, CardType::LIVE_CARD, /*num_channels=*/0, /*active=*/true);  // Type and channels are ignored.
 	global_audio_mixer->trigger_state_changed_callback();
 
 	return ALSAPool::ProbeResult::SUCCESS;
