@@ -189,7 +189,7 @@ void CEFCapture::start_bm_capture()
 
 		CefWindowInfo window_info;
 		window_info.SetAsWindowless(0);
-		browser = CefBrowserHost::CreateBrowserSync(window_info, cef_client, start_url, browser_settings, nullptr);
+		browser = CefBrowserHost::CreateBrowserSync(window_info, cef_client, start_url, browser_settings, nullptr, nullptr);
 		for (function<void()> &task : deferred_tasks) {
 			task();
 		}
