@@ -84,6 +84,9 @@ DeckLinkOutput::DeckLinkOutput(ResourcePool *resource_pool, QSurface *surface, u
 
 bool DeckLinkOutput::set_device(IDeckLink *decklink)
 {
+	if (decklink->QueryInterface(IID_IDeckLinkInput, (void**)&input) != S_OK) {
+		input = nullptr;
+	}
 	if (decklink->QueryInterface(IID_IDeckLinkOutput, (void**)&output) != S_OK) {
 		fprintf(stderr, "Warning: Card %u has no outputs\n", card_index);
 		return false;
@@ -151,7 +154,7 @@ void DeckLinkOutput::start_output(uint32_t mode, int64_t base_pts)
 		fprintf(stderr, "Failed to set video output connection for card %u\n", card_index);
 		abort();
 	}
-	if (config->SetFlag(bmdDeckLinkConfigUse1080pNotPsF, true) != S_OK) {
+	if (config->SetFlag(bmdDeckLinkConfigOutput1080pAsPsF, true) != S_OK) {
 		fprintf(stderr, "Failed to set PsF flag for card\n");
 		abort();
 	}
@@ -193,6 +196,15 @@ void DeckLinkOutput::start_output(uint32_t mode, int64_t base_pts)
 
 	display_mode->Release();
 
+	if (input != nullptr) {
+		if (input->DisableVideoInput() != S_OK) {
+			fprintf(stderr, "Warning: Failed to disable video input for card %d\n", card_index);
+		}
+		if (input->DisableAudioInput() != S_OK) {
+			fprintf(stderr, "Warning: Failed to disable audio input for card %d\n", card_index);
+		}
+	}
+
 	HRESULT result = output->EnableVideoOutput(mode, bmdVideoOutputFlagDefault);
 	if (result != S_OK) {
 		fprintf(stderr, "Couldn't enable output with error 0x%x\n", result);
@@ -248,6 +260,15 @@ void DeckLinkOutput::end_output()
 			frame_freelist.pop();
 		}
 	}
+
+	if (input != nullptr) {
+		input->Release();
+		input = nullptr;
+	}
+	if (output != nullptr) {
+		output->Release();
+		output = nullptr;
+	}
 }
 
 void DeckLinkOutput::send_frame(GLuint y_tex, GLuint cbcr_tex, YCbCrLumaCoefficients output_ycbcr_coefficients, const vector<RefCountedFrame> &input_frames, int64_t pts, int64_t duration)
@@ -462,12 +483,12 @@ HRESULT DeckLinkOutput::ScheduledFrameCompleted(/* in */ IDeckLinkVideoFrame *co
 		++metric_decklink_output_completed_frames_late;
 		break;
 	case bmdOutputFrameDropped:
-		fprintf(stderr, "Output frame was dropped (pts=%" PRId64 "ld)\n", frame->pts);
+		fprintf(stderr, "Output frame was dropped (pts=%" PRId64 ")\n", frame->pts);
 		fprintf(stderr, "Consider increasing --output-buffer-frames if this persists.\n");
 		++metric_decklink_output_completed_frames_dropped;
 		break;
 	case bmdOutputFrameFlushed:
-		fprintf(stderr, "Output frame was flushed (pts=%" PRId64 "ld)\n", frame->pts);
+		fprintf(stderr, "Output frame was flushed (pts=%" PRId64 ")\n", frame->pts);
 		++metric_decklink_output_completed_frames_flushed;
 		break;
 	default:
