@@ -5,7 +5,7 @@
 
 #define MAX_FPS 60
 #define FAKE_FPS 25  // Must be an integer.
-#define MAX_VIDEO_CARDS 16
+// #define MAX_VIDEO_CARDS 16  // defined in shared_defs.h.
 #define MAX_ALSA_CARDS 16
 #define MAX_BUSES 256  // Audio buses.
 
@@ -20,6 +20,7 @@
 #define LOCAL_DUMP_SUFFIX ".nut"
 #define DEFAULT_STREAM_MUX_NAME "nut"  // Only for HTTP. Local dump guesses from LOCAL_DUMP_SUFFIX.
 #define DEFAULT_HTTPD_PORT 9095
+#define DEFAULT_SRT_PORT 9710
 
 #include "shared/shared_defs.h"
 
