@@ -228,7 +228,7 @@ shared_ptr<const ImageInput::Image> ImageInput::load_image_raw(const string &pat
 	glBindTexture(GL_TEXTURE_2D, 0);
 	check_error();
 
-	shared_ptr<Image> image(new Image{unsigned(frame->width), unsigned(frame->height), RefCountedTexture(new GLuint(tex)), last_modified});
+	shared_ptr<Image> image(new Image{unsigned(frame->width), unsigned(frame->height), UniqueTexture(new GLuint(tex)), last_modified});
 	return image;
 }
 
