@@ -47,11 +47,12 @@ int write_packet(void *opaque, uint8_t *buf, int buf_size, AVIODataMarkerType ty
 		type = AVIO_DATA_MARKER_SYNC_POINT;
 	}
 
+	HTTPD::StreamID stream_id{ HTTPD::MAIN_STREAM, 0 };
 	if (type == AVIO_DATA_MARKER_HEADER) {
 		stream_mux_header.append((char *)buf, buf_size);
-		httpd->set_header(HTTPD::MAIN_STREAM, stream_mux_header);
+		httpd->set_header(stream_id, stream_mux_header);
 	} else {
-		httpd->add_data(HTTPD::MAIN_STREAM, (char *)buf, buf_size, type == AVIO_DATA_MARKER_SYNC_POINT, time, AVRational{ AV_TIME_BASE, 1 });
+		httpd->add_data(stream_id, (char *)buf, buf_size, type == AVIO_DATA_MARKER_SYNC_POINT, time, AVRational{ AV_TIME_BASE, 1 });
 	}
 	return buf_size;
 }
@@ -180,7 +181,7 @@ int main(int argc, char *argv[])
 		usage(PROGRAM_KAERU);
 		abort();
 	}
-	global_flags.num_cards = 1;  // For latency metrics.
+	global_flags.max_num_cards = 1;  // For latency metrics.
 
 #if LIBAVFORMAT_VERSION_INT < AV_VERSION_INT(58, 9, 100)
 	av_register_all();
