@@ -23,11 +23,13 @@
 #include <thread>
 #include <vector>
 
+#include <movit/effect.h>
 #include <movit/image_format.h>
 
 #include "audio_mixer.h"
 #include "bmusb/bmusb.h"
 #include "defs.h"
+#include "ffmpeg_capture.h"
 #include "shared/httpd.h"
 #include "input_state.h"
 #include "libusb.h"
@@ -157,7 +159,7 @@ private:
 class Mixer {
 public:
 	// The surface format is used for offscreen destinations for OpenGL contexts we need.
-	Mixer(const QSurfaceFormat &format, unsigned num_cards);
+	Mixer(const QSurfaceFormat &format);
 	~Mixer();
 	void start();
 	void quit();
@@ -250,14 +252,14 @@ public:
 		return theme->get_channel_color(channel);
 	}
 
-	int get_channel_signal(unsigned channel) const
+	int map_channel_to_signal(unsigned channel) const
 	{
-		return theme->get_channel_signal(channel);
+		return theme->map_channel_to_signal(channel);
 	}
 
-	int map_signal(unsigned channel)
+	int map_signal_to_card(int signal)
 	{
-		return theme->map_signal(channel);
+		return theme->map_signal_to_card(signal);
 	}
 
 	unsigned get_master_clock() const
@@ -302,10 +304,8 @@ public:
 		should_cut = true;
 	}
 
-	unsigned get_num_cards() const { return num_cards; }
-
 	std::string get_card_description(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_description();
 	}
 
@@ -315,7 +315,7 @@ public:
 	// the card's actual name.
 	std::string get_output_card_description(unsigned card_index) const {
 		assert(card_can_be_used_as_output(card_index));
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		if (cards[card_index].parked_capture) {
 			return cards[card_index].parked_capture->get_description();
 		} else {
@@ -324,59 +324,87 @@ public:
 	}
 
 	bool card_can_be_used_as_output(unsigned card_index) const {
-		assert(card_index < num_cards);
-		return cards[card_index].output != nullptr;
+		assert(card_index < MAX_VIDEO_CARDS);
+		return cards[card_index].output != nullptr && cards[card_index].capture != nullptr;
+	}
+
+	bool card_is_cef(unsigned card_index) const {
+		assert(card_index < MAX_VIDEO_CARDS);
+		return cards[card_index].type == CardType::CEF_INPUT;
 	}
 
 	bool card_is_ffmpeg(unsigned card_index) const {
-		assert(card_index < num_cards + num_video_inputs);
-		return cards[card_index].type == CardType::FFMPEG_INPUT;
+		assert(card_index < MAX_VIDEO_CARDS);
+		if (cards[card_index].type != CardType::FFMPEG_INPUT) {
+			return false;
+		}
+#ifdef HAVE_SRT
+		// SRT inputs are more like regular inputs than FFmpeg inputs,
+		// so show them as such. (This allows the user to right-click
+		// to select a different input.)
+		return static_cast<FFmpegCapture *>(cards[card_index].capture.get())->get_srt_sock() == -1;
+#else
+		return true;
+#endif
+	}
+
+	bool card_is_active(unsigned card_index) const {
+		assert(card_index < MAX_VIDEO_CARDS);
+		std::lock_guard<std::mutex> lock(card_mutex);
+		return cards[card_index].capture != nullptr;
+	}
+
+	void force_card_active(unsigned card_index)
+	{
+		// handle_hotplugged_cards() will pick this up.
+		std::lock_guard<std::mutex> lock(card_mutex);
+		cards[card_index].force_active = true;
 	}
 
 	std::map<uint32_t, bmusb::VideoMode> get_available_video_modes(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_available_video_modes();
 	}
 
 	uint32_t get_current_video_mode(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_current_video_mode();
 	}
 
 	void set_video_mode(unsigned card_index, uint32_t mode) {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		cards[card_index].capture->set_video_mode(mode);
 	}
 
 	void start_mode_scanning(unsigned card_index);
 
 	std::map<uint32_t, std::string> get_available_video_inputs(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_available_video_inputs();
 	}
 
 	uint32_t get_current_video_input(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_current_video_input();
 	}
 
 	void set_video_input(unsigned card_index, uint32_t input) {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		cards[card_index].capture->set_video_input(input);
 	}
 
 	std::map<uint32_t, std::string> get_available_audio_inputs(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_available_audio_inputs();
 	}
 
 	uint32_t get_current_audio_input(unsigned card_index) const {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		return cards[card_index].capture->get_current_audio_input();
 	}
 
 	void set_audio_input(unsigned card_index, uint32_t input) {
-		assert(card_index < num_cards);
+		assert(card_index < MAX_VIDEO_CARDS);
 		cards[card_index].capture->set_audio_input(input);
 	}
 
@@ -432,17 +460,15 @@ public:
 private:
 	struct CaptureCard;
 
-	enum class CardType {
-		LIVE_CARD,
-		FAKE_CAPTURE,
-		FFMPEG_INPUT,
-		CEF_INPUT,
-	};
-	void configure_card(unsigned card_index, bmusb::CaptureInterface *capture, CardType card_type, DeckLinkOutput *output);
+	void configure_card(unsigned card_index, bmusb::CaptureInterface *capture, CardType card_type, DeckLinkOutput *output, bool is_srt_card);
 	void set_output_card_internal(int card_index);  // Should only be called from the mixer thread.
 	void bm_frame(unsigned card_index, uint16_t timecode,
 		bmusb::FrameAllocator::Frame video_frame, size_t video_offset, bmusb::VideoFormat video_format,
 		bmusb::FrameAllocator::Frame audio_frame, size_t audio_offset, bmusb::AudioFormat audio_format);
+	void upload_texture_for_frame(
+	        int field, bmusb::VideoFormat video_format,
+		size_t y_offset, size_t cbcr_offset, size_t video_offset,
+		PBOFrameAllocator::Userdata *userdata);
 	void bm_hotplug_add(libusb_device *dev);
 	void bm_hotplug_remove(unsigned card_index);
 	void place_rectangle(movit::Effect *resample_effect, movit::Effect *padding_effect, float x0, float y0, float x1, float y1);
@@ -453,13 +479,16 @@ private:
 	void render_one_frame(int64_t duration);
 	void audio_thread_func();
 	void release_display_frame(DisplayFrame *frame);
+#ifdef HAVE_SRT
+	void start_srt();
+#endif
 	double pts() { return double(pts_int) / TIMEBASE; }
 	void trim_queue(CaptureCard *card, size_t safe_queue_length);
 	std::pair<std::string, std::string> get_channels_json();
 	std::pair<std::string, std::string> get_channel_color_http(unsigned channel_idx);
 
 	HTTPD httpd;
-	unsigned num_cards, num_video_inputs, num_html_inputs = 0;
+	unsigned num_video_inputs, num_html_inputs = 0;
 
 	QSurface *mixer_surface, *h264_encoder_surface, *decklink_output_surface, *image_update_surface;
 	std::unique_ptr<movit::ResourcePool> resource_pool;
@@ -500,11 +529,24 @@ private:
 	// frame rate is integer, will always stay zero.
 	unsigned fractional_samples = 0;
 
+	// Monotonic counter that lets us know which slot was last turned into
+	// a fake capture. Used for SRT re-plugging.
+	unsigned fake_capture_counter = 0;
+
 	mutable std::mutex card_mutex;
 	bool has_bmusb_thread = false;
 	struct CaptureCard {
+		// If nullptr, the card is inactive, and will be hidden in the UI.
+		// Only fake capture cards can be inactive.
 		std::unique_ptr<bmusb::CaptureInterface> capture;
+		// If true, card must always be active (typically because it's one of the
+		// first cards, or because the theme has explicitly asked for it).
+		bool force_active = false;
 		bool is_fake_capture;
+		// If is_fake_capture is true, contains a monotonic timer value for when
+		// it was last changed. Otherwise undefined. Used for SRT re-plugging.
+		int fake_capture_counter;
+		std::string last_srt_stream_id = "<default, matches nothing>";  // Used for SRT re-plugging.
 		CardType type;
 		std::unique_ptr<DeckLinkOutput> output;
 
@@ -534,15 +576,15 @@ private:
 			int64_t length;  // In TIMEBASE units.
 			bool interlaced;
 			unsigned field;  // Which field (0 or 1) of the frame to use. Always 0 for progressive.
-			std::function<void()> upload_func;  // Needs to be called to actually upload the texture to OpenGL.
+			bool texture_uploaded = false;
 			unsigned dropped_frames = 0;  // Number of dropped frames before this one.
 			std::chrono::steady_clock::time_point received_timestamp = std::chrono::steady_clock::time_point::min();
+			movit::RGBTriplet neutral_color{1.0f, 1.0f, 1.0f};
 
-			// Used for MJPEG encoding. (upload_func packs everything it needs
-			// into the functor, but would otherwise also use these.)
+			// Used for MJPEG encoding, and texture upload.
 			// width=0 or height=0 means a broken frame, ie., do not upload.
 			bmusb::VideoFormat video_format;
-			size_t y_offset, cbcr_offset;
+			size_t video_offset, y_offset, cbcr_offset;
 		};
 		std::deque<NewFrame> new_frames;
 		std::condition_variable new_frames_changed;  // Set whenever new_frames is changed.
@@ -571,10 +613,58 @@ private:
 		std::atomic<int64_t> metric_input_frame_rate_nom{-1};
 		std::atomic<int64_t> metric_input_frame_rate_den{-1};
 		std::atomic<int64_t> metric_input_sample_rate_hz{-1};
+
+		// SRT metrics.
+		std::atomic<double> metric_srt_uptime_seconds{0.0 / 0.0};
+		std::atomic<double> metric_srt_send_duration_seconds{0.0 / 0.0};
+		std::atomic<int64_t> metric_srt_sent_bytes{-1};
+		std::atomic<int64_t> metric_srt_received_bytes{-1};
+		std::atomic<int64_t> metric_srt_sent_packets_normal{-1};
+		std::atomic<int64_t> metric_srt_received_packets_normal{-1};
+		std::atomic<int64_t> metric_srt_sent_packets_lost{-1};
+		std::atomic<int64_t> metric_srt_received_packets_lost{-1};
+		std::atomic<int64_t> metric_srt_sent_packets_retransmitted{-1};
+		std::atomic<int64_t> metric_srt_sent_bytes_retransmitted{-1};
+		std::atomic<int64_t> metric_srt_sent_packets_ack{-1};
+		std::atomic<int64_t> metric_srt_received_packets_ack{-1};
+		std::atomic<int64_t> metric_srt_sent_packets_nak{-1};
+		std::atomic<int64_t> metric_srt_received_packets_nak{-1};
+		std::atomic<int64_t> metric_srt_sent_packets_dropped{-1};
+		std::atomic<int64_t> metric_srt_received_packets_dropped{-1};
+		std::atomic<int64_t> metric_srt_sent_bytes_dropped{-1};
+		std::atomic<int64_t> metric_srt_received_bytes_dropped{-1};
+		std::atomic<int64_t> metric_srt_received_packets_undecryptable{-1};
+		std::atomic<int64_t> metric_srt_received_bytes_undecryptable{-1};
+
+		std::atomic<int64_t> metric_srt_filter_received_extra_packets{-1};
+		std::atomic<int64_t> metric_srt_filter_received_rebuilt_packets{-1};
+		std::atomic<int64_t> metric_srt_filter_received_lost_packets{-1};
+
+		std::atomic<double> metric_srt_packet_sending_period_seconds{0.0 / 0.0};
+		std::atomic<int64_t> metric_srt_flow_window_packets{-1};
+		std::atomic<int64_t> metric_srt_congestion_window_packets{-1};
+		std::atomic<int64_t> metric_srt_flight_size_packets{-1};
+		std::atomic<double> metric_srt_rtt_seconds{0.0 / 0.0};
+		std::atomic<double> metric_srt_estimated_bandwidth_bits_per_second{0.0 / 0.0};
+		std::atomic<double> metric_srt_bandwidth_ceiling_bits_per_second{0.0 / 0.0};
+		std::atomic<int64_t> metric_srt_send_buffer_available_bytes{-1};
+		std::atomic<int64_t> metric_srt_receive_buffer_available_bytes{-1};
+		std::atomic<int64_t> metric_srt_mss_bytes{-1};
+		std::atomic<int64_t> metric_srt_sender_unacked_packets{-1};
+		std::atomic<int64_t> metric_srt_sender_unacked_bytes{-1};
+		std::atomic<double> metric_srt_sender_unacked_timespan_seconds{0.0 / 0.0};
+		std::atomic<double> metric_srt_sender_delivery_delay_seconds{0.0 / 0.0};
+		std::atomic<int64_t> metric_srt_receiver_unacked_packets{-1};
+		std::atomic<int64_t> metric_srt_receiver_unacked_bytes{-1};
+		std::atomic<double> metric_srt_receiver_unacked_timespan_seconds{0.0 / 0.0};
+		std::atomic<double> metric_srt_receiver_delivery_delay_seconds{0.0 / 0.0};
+		std::atomic<int64_t> metric_srt_filter_sent_packets{-1};
+
 	};
 	JitterHistory output_jitter_history;
 	CaptureCard cards[MAX_VIDEO_CARDS];  // Protected by <card_mutex>.
 	YCbCrInterpretation ycbcr_interpretation[MAX_VIDEO_CARDS];  // Protected by <card_mutex>.
+	movit::RGBTriplet last_received_neutral_color[MAX_VIDEO_CARDS];  // Used by the mixer thread only. Constructor-initialiezd.
 	std::unique_ptr<AudioMixer> audio_mixer;  // Same as global_audio_mixer (see audio_mixer.h).
 	bool input_card_is_master_clock(unsigned card_index, unsigned master_card_index) const;
 	struct OutputFrameInfo {
@@ -586,12 +676,22 @@ private:
 	};
 	OutputFrameInfo get_one_frame_from_each_card(unsigned master_card_index, bool master_card_is_output, CaptureCard::NewFrame new_frames[MAX_VIDEO_CARDS], bool has_new_frame[MAX_VIDEO_CARDS], std::vector<int32_t> raw_audio[MAX_VIDEO_CARDS]);
 
+#ifdef HAVE_SRT
+	void update_srt_stats(int srt_sock, Mixer::CaptureCard *card);
+#endif
+
+	std::string description_for_card(unsigned card_index);
+	static bool is_srt_card(const CaptureCard *card);
+
 	InputState input_state;
 
 	// Cards we have been noticed about being hotplugged, but haven't tried adding yet.
 	// Protected by its own mutex.
 	std::mutex hotplug_mutex;
 	std::vector<libusb_device *> hotplugged_cards;
+#ifdef HAVE_SRT
+	std::vector<int> hotplugged_srt_cards;
+#endif
 
 	class OutputChannel {
 	public:
@@ -624,6 +724,9 @@ private:
 
 	std::thread mixer_thread;
 	std::thread audio_thread;
+#ifdef HAVE_SRT
+	std::thread srt_thread;
+#endif
 	std::atomic<bool> should_quit{false};
 	std::atomic<bool> should_cut{false};
 
