@@ -20,7 +20,8 @@
 #include "shared/timebase.h"
 #include "print_latency.h"
 #include "shared/ref_counted_gl_sync.h"
-#include "va_display_with_cleanup.h"
+#include "shared/va_display.h"
+#include "v4l_output.h"
 
 #define SURFACE_NUM 16 /* 16 surfaces for source YUV */
 #define MAX_NUM_REF1 16 // Seemingly a hardware-fixed value, not related to SURFACE_NUM
@@ -171,6 +172,7 @@ private:
 	std::unique_ptr<AudioEncoder> file_audio_encoder;
 
 	X264Encoder *x264_encoder;  // nullptr if not using x264.
+	std::unique_ptr<V4LOutput> v4l_output;  // nullptr if not using V4L2 output.
 
 	Mux* stream_mux = nullptr;  // To HTTP.
 	std::unique_ptr<Mux> file_mux;  // To local disk.
