@@ -2,6 +2,7 @@
 #define _THEME_H 1
 
 #include <lua.hpp>
+#include <movit/effect.h>
 #include <movit/flat_input.h>
 #include <movit/ycbcr_input.h>
 #include <stdbool.h>
@@ -39,6 +40,7 @@ enum EffectType {
 
 	IDENTITY_EFFECT,
 	WHITE_BALANCE_EFFECT,
+	AUTO_WHITE_BALANCE_EFFECT,  // Same as WHITE_BALANCE_EFFECT, but sets its value automatically.
 	RESAMPLE_EFFECT,
 	PADDING_EFFECT,
 	INTEGRAL_PADDING_EFFECT,
@@ -76,13 +78,14 @@ struct InputStateInfo {
 	unsigned last_width[MAX_VIDEO_CARDS], last_height[MAX_VIDEO_CARDS];
 	bool last_interlaced[MAX_VIDEO_CARDS], last_has_signal[MAX_VIDEO_CARDS], last_is_connected[MAX_VIDEO_CARDS];
 	unsigned last_frame_rate_nom[MAX_VIDEO_CARDS], last_frame_rate_den[MAX_VIDEO_CARDS];
+	bmusb::PixelFormat last_pixel_format[MAX_VIDEO_CARDS];
 	bool has_last_subtitle[MAX_VIDEO_CARDS];
 	std::string last_subtitle[MAX_VIDEO_CARDS];
 };
 
 class Theme {
 public:
-	Theme(const std::string &filename, const std::vector<std::string> &search_dirs, movit::ResourcePool *resource_pool, unsigned num_cards);
+	Theme(const std::string &filename, const std::vector<std::string> &search_dirs, movit::ResourcePool *resource_pool);
 	~Theme();
 
 	struct Chain {
@@ -97,14 +100,18 @@ public:
 	Chain get_chain(unsigned num, float t, unsigned width, unsigned height, const InputState &input_state);
 
 	int get_num_channels() const { return num_channels; }
-	int map_signal(int signal_num);
-	void set_signal_mapping(int signal_num, int card_num);
+	int map_signal_to_card(int signal_num);
+	void set_signal_mapping(int signal_num, int card_idx);
 	std::string get_channel_name(unsigned channel);
-	int get_channel_signal(unsigned channel);
+	int map_channel_to_signal(unsigned channel);
 	bool get_supports_set_wb(unsigned channel);
-	void set_wb(unsigned channel, double r, double g, double b);
+	void set_wb(unsigned channel, float r, float g, float b);
+	void set_wb_for_card(int card_idx, float r, float g, float b);
+	movit::RGBTriplet get_white_balance_for_card(int card_idx);
 	std::string get_channel_color(unsigned channel);
 
+	std::unordered_map<int, movit::RGBTriplet> white_balance_for_card;
+
 	std::vector<std::string> get_transition_names(float t);
 
 	void transition_clicked(int transition_num, float t);
@@ -185,11 +192,17 @@ public:
 
 	std::string format_status_line(const std::string &disk_space_left_text, double file_length_seconds);
 
+	// Signal that the given card is going away and will not be replaced
+	// with a fake capture card, so remove all connections to it so that
+	// they don't automatically come back on the next frame.
+	void remove_card(unsigned card_index);
+
 private:
 	void register_globals();
 	void register_class(const char *class_name, const luaL_Reg *funcs, EffectType effect_type = NO_EFFECT_TYPE);
 	int set_theme_menu(lua_State *L);
 	Chain get_chain_from_effect_chain(movit::EffectChain *effect_chain, unsigned num, const InputState &input_state);
+	void call_lua_wb_callback(unsigned channel, float r, float g, float b);
 
 	std::string theme_path;
 
@@ -198,7 +211,6 @@ private:
 	const InputState *input_state = nullptr;  // Protected by <m>. Only set temporarily, during chain setup.
 	movit::ResourcePool *resource_pool;
 	int num_channels = -1;
-	unsigned num_cards;
 	bool startup_finished = false;
 
 	std::mutex map_m;
@@ -248,7 +260,7 @@ public:
 	LiveInputWrapper(Theme *theme, movit::EffectChain *chain, bmusb::PixelFormat pixel_format, bool override_bounce, bool deinterlace, bool user_connectable);
 
 	bool connect_signal(int signal_num);  // Must be called with the theme's <m> lock held, since it accesses theme->input_state. Returns false on error.
-	void connect_signal_raw(int signal_num, const InputState &input_state);
+	void connect_card(int signal_num, const InputState &input_state);
 	movit::Effect *get_effect() const
 	{
 		if (deinterlace) {
