@@ -232,7 +232,7 @@ void X264Encoder::init_x264()
 	x264 = dyn.x264_encoder_open(&param);
 	if (x264 == nullptr) {
 		fprintf(stderr, "ERROR: x264 initialization failed.\n");
-		exit(1);
+		abort();
 	}
 
 	if (global_flags.x264_speedcontrol) {
